
--
-- Create triggers for Quadrige2 (need after ReefDb changes)
--
--  project : ${pom.name}
--  version : ${pom.version} for ${pom.env}
--      env : ${pom.env} - ${pom.profil}
--     date : ${pom.date.fr}
--
--  Copyright Ifremer 2014
-- 
-- 18/11/2014 BL  Creation
-- 12/10/2015 BL  Add trigger TBIU_APPLIED_PERIOD
-- 16/11/2015 BL  Add trigger on PMFM, MONITORING_LOCATION, LOCATION, ...
-- 19/11/2015 BL  Add trigger on deletion (PROGRAMME, ...)
-- 09/12/2015 BL  Add special trigger for deletion on MONITORING_LOCATION (use a special object type)
-- 29/02/2016 BL  Add trigger TBIU_SANDRE_PMFMU_EXP (fix mantis #29269)
-- 06/04/2016 LP  Add trigger TBIU_SANDRE_PMFMU_IMP
-- 17/05/2016 LP  Add trigger on deletion (CAMPAIGN, OCCASION)
-- 12/10/2016 LP  Update Triggers for some PMFM tables (mantis #32027)
-- 07/04/2017 LP  Add trigger on quser_privilege (mantis #34994)
-- --------------------------------------------------------------------------

-----------------------------------------------------------------------
-- MONITORING_LOCATION
--  * manage new STATUS_CD column
-----------------------------------------------------------------------
create or replace trigger "TBIU_MONITORING_LOCATION" before insert or update
on MONITORING_LOCATION for each row
when (new.STATUS_CD is null)
begin
    :new.STATUS_CD := '1';
end;
/

create or replace trigger TBD_MONITORING_LOCATION before delete
on MONITORING_LOCATION for each row
BEGIN
 BEGIN
  INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.MON_LOC_ID, systimestamp, 'LIEU');
 EXCEPTION
  WHEN OTHERS THEN
   INSERT INTO OBJECT_TYPE (OBJECT_TYPE_CD, OBJECT_TYPE_NM, UPDATE_DT) values('LIEU', 'Lieu de surveillance', systimestamp);
   INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.MON_LOC_ID, systimestamp, 'LIEU');
  END;
END;
/

-----------------------------------------------------------------------
-- PROGRAMME
--  * manage new STATUS_CD column
-----------------------------------------------------------------------
create or replace trigger "TBIU_PROGRAMME" before insert or update
on PROGRAMME for each row
when (new.STATUS_CD is null)
begin
    :new.STATUS_CD := '1';
end;
/

-----------------------------------------------------------------------
-- ALTERNATIVE_TAXON
-----------------------------------------------------------------------
create or replace 
trigger TBD_ALTERNATIVE_TAXON before delete on ALTERNATIVE_TAXON
for each row 
BEGIN 
  BEGIN 
    INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.ALTERN_TAXON_ID, systimestamp, 'ALTERNATIVE_TAXON'); 
    EXCEPTION WHEN OTHERS THEN 
      INSERT INTO OBJECT_TYPE (OBJECT_TYPE_CD, OBJECT_TYPE_NM, UPDATE_DT) values('ALTERNATIVE_TAXON', 'Table ALTERNATIVE_TAXON', systimestamp); 
      INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.ALTERN_TAXON_ID, systimestamp, 'ALTERNATIVE_TAXON'); 
  END;
END;
/

-- Insert/update
create or replace trigger "TBIU_ALTERNATIVE_TAXON" before insert
on ALTERNATIVE_TAXON for each row
when (new.ALTERN_TAXON_ID is null)
begin
    -- Trigger permettant de remplir les colonnes obligatoires manquantes
    -- quand les insertions sont faites par le serveur de synchro
    select
      SEQ_ALTERN_TAXON_ID.nextval into :new.ALTERN_TAXON_ID
    from DUAL;
end;
/

-----------------------------------------------------------------------
-- APPLIED_STRATEGY
-----------------------------------------------------------------------
create or replace
trigger "TBIU_APPLIED_STRATEGY" before insert
on APPLIED_STRATEGY for each row
when (new.APPLIED_STRAT_ID is null)
begin
  -- Trigger permettant de remplir les colonnes obligatoires manquantes
  -- quand les insertions sont faites par le serveur de synchro
  select
    SEQ_APPLIED_STRAT_ID.nextval into :new.APPLIED_STRAT_ID
  from DUAL;
end;
/

-----------------------------------------------------------------------
-- APPLIED_PERIOD
-----------------------------------------------------------------------
create or replace trigger "TBIU_APPLIED_PERIOD" before insert or update
on APPLIED_PERIOD for each row
begin
  if ((:new.STRAT_ID is null) or (:new.MON_LOC_ID is null)) then
    -- Permet de remplir les colonnes obligatoires manquantes
    -- quand les insertions sont faites par le serveur de synchro
    select STRAT_ID, MON_LOC_ID into :new.STRAT_ID, :new.MON_LOC_ID
    from APPLIED_STRATEGY
    where APPLIED_STRAT_ID = :new.APPLIED_STRAT_ID ;
  end if;
  
  if (:new.APPLIED_STRAT_ID is null) then
    -- Permet de remplir les colonnes obligatoires manquantes
    -- quand les insertions sont faites par le client/serveur Quadrige2
    select APPLIED_STRAT_ID into :new.APPLIED_STRAT_ID
    from APPLIED_STRATEGY
    where 
      STRAT_ID = :new.STRAT_ID
      AND MON_LOC_ID = :new.MON_LOC_ID;
  end if;
  
end;
/

-----------------------------------------------------------------------
-- CODE_SIZE
-----------------------------------------------------------------------
create or replace 
trigger TBD_CODE_SIZE before delete on CODE_SIZE
for each row 
BEGIN 
  BEGIN 
    INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.CODE_SIZE_ID, systimestamp, 'CODE_SIZE'); 
    EXCEPTION WHEN OTHERS THEN 
      INSERT INTO OBJECT_TYPE (OBJECT_TYPE_CD, OBJECT_TYPE_NM, UPDATE_DT) values('CODE_SIZE', 'Table CODE_SIZE', systimestamp); 
      INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.CODE_SIZE_ID, systimestamp, 'CODE_SIZE'); 
  END;
END;
/

create or replace
trigger "TBI_CODE_SIZE" before insert
on CODE_SIZE for each row
when (new.CODE_SIZE_ID is null)
begin
  select
    SEQ_CODE_SIZE_ID.nextval into :new.CODE_SIZE_ID
  from DUAL;
end;
/

-----------------------------------------------------------------------
-- CONTEXT
--  * manage new STATUS_CD column
-----------------------------------------------------------------------
create or replace
trigger "TBI_CONTEXT" before insert
on CONTEXT for each row
when (new.STATUS_CD is null)
begin
  :new.STATUS_CD := '1';
end;
/

-----------------------------------------------------------------------
-- DEFAULT_VALUE
-----------------------------------------------------------------------
create or replace 
trigger TBD_DEFAULT_VALUE before delete on DEFAULT_VALUE
for each row 
BEGIN 
  BEGIN 
    INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.DEFAULT_VALUE_ID, systimestamp, 'DEFAULT_VALUE'); 
    EXCEPTION WHEN OTHERS THEN 
      INSERT INTO OBJECT_TYPE (OBJECT_TYPE_CD, OBJECT_TYPE_NM, UPDATE_DT) values('DEFAULT_VALUE', 'Table DEFAULT_VALUE', systimestamp); 
      INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.DEFAULT_VALUE_ID, systimestamp, 'DEFAULT_VALUE'); 
  END;
END;
/

create or replace
trigger "TBI_DEFAULT_VALUE" before insert
on DEFAULT_VALUE for each row
when (new.DEFAULT_VALUE_ID is null)
begin
  select
    SEQ_DEFAULT_VALUE_ID.nextval into :new.DEFAULT_VALUE_ID
  from DUAL;
end;
/

-----------------------------------------------------------------------
-- PMFM_STRATEGY
-----------------------------------------------------------------------
create or replace 
trigger "TBIU_PMFM_STRATEGY" before insert or update
on PMFM_STRATEGY for each row
begin
  -- Trigger permettant de remplir les colonnes obligatoires manquantes
  -- quand les insertions sont faites par le serveur de synchro
  if (:new.PMFM_ID is not null AND (:new.PAR_CD is null OR :new.MATRIX_ID is null OR :new.FRACTION_ID is null OR :new.METHOD_ID is null)) then
    BEGIN
      select
        PAR_CD, MATRIX_ID, FRACTION_ID, METHOD_ID
      into :new.PAR_CD, :new.MATRIX_ID , :new.FRACTION_ID, :new.METHOD_ID
      from PMFM
      where PMFM_ID = :new.PMFM_ID;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        -- PMFM not found
        raise_application_error(-20101, 'PMFM_ID='|| :new.PMFM_ID ||' not exists in table PMFM.');
    END;
  end if;
  
  if (:new.PMFM_STRAT_ID is null) then
    select
      SEQ_PMFM_STRAT_ID.nextval
      into :new.PMFM_STRAT_ID
    from DUAL;
  end if;
  
   -- Make sure PMFM_ID is filled (need for ReefDB synchro-server)
  if (:new.PMFM_ID is null) then
    select
      PMFM_ID into :new.PMFM_ID
    from PMFM
    where 
      PAR_CD = :new.PAR_CD
      and MATRIX_ID = :new.MATRIX_ID
      and FRACTION_ID = :new.FRACTION_ID
      and METHOD_ID = :new.METHOD_ID;
  end if;

  -- Ensure that PMFM tuple is valid (Mantis #320027)
  if (:new.PMFM_ID is not null AND
      :new.PAR_CD is not null AND
      :new.MATRIX_ID is not null AND
      :new.FRACTION_ID is not null AND
      :new.METHOD_ID is not null) then
    DECLARE
      pmfmId PMFM.PMFM_ID%TYPE;
    BEGIN
      SELECT PMFM_ID INTO pmfmId FROM PMFM
      WHERE PMFM_ID = :new.PMFM_ID
      AND PAR_CD = :new.PAR_CD
      AND MATRIX_ID = :new.MATRIX_ID
      AND FRACTION_ID = :new.FRACTION_ID
      AND METHOD_ID = :new.METHOD_ID;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        raise_application_error(-20101, 'PMFM_ID='||:new.PMFM_ID||', PAR_CD='||:new.PAR_CD||', MATRIX_ID='||:new.MATRIX_ID||', FRACTION_ID='||:new.FRACTION_ID||', METHOD_ID='||:new.METHOD_ID||' not exists in table PMFM.');
    END;
  end if;

end;
/


-----------------------------------------------------------------------
-- PMFM
-----------------------------------------------------------------------
-- Insert
create or replace 
trigger TBI_PMFM before insert 
on PMFM for each row 
when (new.PMFM_ID is null)
begin
  select SEQ_PMFM_ID.nextval into :new.PMFM_ID from dual;
end;
/

-- Delete
create or replace 
trigger TBD_PMFM before delete 
on PMFM for each row 
begin
  insert into DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD)
  values(SEQ_DEL_ITEM_HIST_ID.nextval, :old.PMFM_ID, systimestamp, 'PMFM');
end;
/

-----------------------------------------------------------------------
-- PMFM_APPLIED_STRATEGY
-----------------------------------------------------------------------
create or replace
trigger "TBIU_PMFM_APPLIED_STRATEGY" before insert or update
on PMFM_APPLIED_STRATEGY for each row
begin
  -- Trigger permettant de remplir les colonnes obligatoires manquantes
  -- quand les insertions sont faites par le serveur de synchro
  IF ((:new.PMFM_STRAT_ID IS NOT NULL)
     AND ((:new.STRAT_ID IS NULL) or (:new.PAR_CD is null) OR (:new.MATRIX_ID is null) OR (:new.FRACTION_ID is null) OR (:new.METHOD_ID is null))) then
    select
      STRAT_ID, PAR_CD, MATRIX_ID, FRACTION_ID, METHOD_ID
    into :new.STRAT_ID, :new.PAR_CD, :new.MATRIX_ID, :new.FRACTION_ID, :new.METHOD_ID
    from PMFM_STRATEGY
    where PMFM_STRAT_ID = :new.PMFM_STRAT_ID;
  END IF;
  
  IF ((:new.PMFM_STRAT_ID IS NULL)
      AND (:new.STRAT_ID IS NOT NULL) AND (:new.PAR_CD is NOT null) AND (:new.MATRIX_ID is NOT null) AND (:new.FRACTION_ID is NOT null) AND (:new.METHOD_ID is NOT null)) then
    select
      PMFM_STRAT_ID
      into :new.PMFM_STRAT_ID
    from PMFM_STRATEGY
    where 
      STRAT_ID = :new.STRAT_ID
      and PAR_CD = :new.PAR_CD
      and MATRIX_ID = :new.MATRIX_ID
      and FRACTION_ID = :new.FRACTION_ID
      and METHOD_ID = :new.METHOD_ID;
  END IF;

  -- Ensure that PMFM tuple is valid (Mantis #320027)
  if (:new.PMFM_STRAT_ID is not null AND
      :new.PAR_CD is not null AND
      :new.MATRIX_ID is not null AND
      :new.FRACTION_ID is not null AND
      :new.METHOD_ID is not null) then
    DECLARE
      pmfmStratId PMFM_STRATEGY.PMFM_STRAT_ID%TYPE;
    BEGIN
      SELECT PMFM_STRAT_ID INTO pmfmStratId FROM PMFM_STRATEGY
      WHERE PMFM_STRAT_ID = :new.PMFM_STRAT_ID
      AND PAR_CD = :new.PAR_CD
      AND MATRIX_ID = :new.MATRIX_ID
      AND FRACTION_ID = :new.FRACTION_ID
      AND METHOD_ID = :new.METHOD_ID;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        raise_application_error(-20101, 'PMFM_STRAT_ID='||:new.PMFM_STRAT_ID||', PAR_CD='||:new.PAR_CD||', MATRIX_ID='||:new.MATRIX_ID||', FRACTION_ID='||:new.FRACTION_ID||', METHOD_ID='||:new.METHOD_ID||' not exists in table PMFM_STRATEGY.');
    END;
  end if;

  IF ((:new.APPLIED_STRAT_ID IS NOT NULL)
     AND ((:new.STRAT_ID IS NULL) or (:new.MON_LOC_ID is null))) then
    select
      STRAT_ID, MON_LOC_ID
    into :new.STRAT_ID, :new.MON_LOC_ID
    from APPLIED_STRATEGY
    where APPLIED_STRAT_ID = :new.APPLIED_STRAT_ID;
  END IF;
  
  IF ((:new.APPLIED_STRAT_ID IS NULL) 
    AND (:new.STRAT_ID IS NOT NULL) AND (:new.MON_LOC_ID IS NOT NULL)) then
    select
      APPLIED_STRAT_ID 
      into :new.APPLIED_STRAT_ID
    from APPLIED_STRATEGY
    where 
      STRAT_ID = :new.STRAT_ID
      AND MON_LOC_ID = :new.MON_LOC_ID;
  END IF;
end;
/
      
-----------------------------------------------------------------------
-- PMFM_QUAL_VALUE
-----------------------------------------------------------------------
-- Insert
CREATE OR REPLACE TRIGGER TBIU_PMFM_QUAL_VALUE before insert or update
on PMFM_QUAL_VALUE for each row
begin

  if (:new.PMFM_ID is null) then
    select
      PMFM_ID into :new.PMFM_ID
    from PMFM
    where
      PAR_CD = :new.PAR_CD
      and MATRIX_ID = :new.MATRIX_ID
      and FRACTION_ID = :new.FRACTION_ID
      and METHOD_ID = :new.METHOD_ID;
  end if;

  -- Ensure that PMFM tuple is valid (Mantis #320027)
  if (:new.PMFM_ID is not null AND
      :new.PAR_CD is not null AND
      :new.MATRIX_ID is not null AND
      :new.FRACTION_ID is not null AND
      :new.METHOD_ID is not null) then
    DECLARE
      pmfmId PMFM.PMFM_ID%TYPE;
    BEGIN
      SELECT PMFM_ID INTO pmfmId FROM PMFM
      WHERE PMFM_ID = :new.PMFM_ID
      AND PAR_CD = :new.PAR_CD
      AND MATRIX_ID = :new.MATRIX_ID
      AND FRACTION_ID = :new.FRACTION_ID
      AND METHOD_ID = :new.METHOD_ID;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        raise_application_error(-20101, 'PMFM_ID='||:new.PMFM_ID||', PAR_CD='||:new.PAR_CD||', MATRIX_ID='||:new.MATRIX_ID||', FRACTION_ID='||:new.FRACTION_ID||', METHOD_ID='||:new.METHOD_ID||' not exists in table PMFM.');
    END;
  end if;

end;
/

-----------------------------------------------------------------------
-- MEASUREMENT
-- LP 03/01/2017: update PMFM_ID if quadruplet changes (Mantis #33435)
-----------------------------------------------------------------------
create or replace 
trigger "TBIU_MEASUREMENT" before insert or update

on MEASUREMENT for each row
declare
	nb_count number;
begin
	-- Trigger permettant d'avoir directement les ID des données IN SITU sur lesquelles sont les résultats
	case :new.OBJECT_TYPE_CD 

    when 'PASS' then
      :new.SURVEY_ID := :new.OBJECT_ID ;
  
    when 'PREL' then
    begin
      :new.SAMPLING_OPER_ID := :new.OBJECT_ID ;
      select count(SURVEY_ID) into nb_count from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.OBJECT_ID ;
      if (nb_count = 1) then
        select SURVEY_ID into :new.SURVEY_ID from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.OBJECT_ID ;
      end if;
    end ;
  
    when 'ECHANT' then
    begin
      :new.SAMPLE_ID := :new.OBJECT_ID ;
      select count(SAMPLING_OPER_ID) into nb_count from SAMPLE where SAMPLE_ID = :new.OBJECT_ID ;
      if (nb_count = 1) then 
        select SAMPLING_OPER_ID into :new.SAMPLING_OPER_ID from SAMPLE where SAMPLE_ID = :new.OBJECT_ID ; 
      end if;
      select count(SURVEY_ID) into nb_count from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.SAMPLING_OPER_ID ;
      if (nb_count = 1) then
        select SURVEY_ID into :new.SURVEY_ID from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.SAMPLING_OPER_ID ;
      end if;
    end ;

	end case;
  
  -- Make sure PMFM_ID is filled and valid (need for ReefDB synchro-server)
  if (:new.PMFM_ID is null
      -- also update PMFM_ID if quadruplet changes (Mantis #33435)
      or :old.PAR_CD != :new.PAR_CD or :old.MATRIX_ID != :new.MATRIX_ID or :old.FRACTION_ID != :new.FRACTION_ID or :old.METHOD_ID != :new.METHOD_ID) then
    begin
      select
        PMFM_ID into :new.PMFM_ID
      from PMFM
      where
        PAR_CD = :new.PAR_CD
        and MATRIX_ID = :new.MATRIX_ID
        and FRACTION_ID = :new.FRACTION_ID
        and METHOD_ID = :new.METHOD_ID;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          raise_application_error(-20101, 'PAR_CD='||:new.PAR_CD||', MATRIX_ID='||:new.MATRIX_ID||', FRACTION_ID='||:new.FRACTION_ID||', METHOD_ID='||:new.METHOD_ID||' not exists in table PMFM.');
    end;
  end if;

end;
/


-----------------------------------------------------------------------
-- MEASUREMENT_FILE
-- LP 03/01/2017: update PMFM_ID if quadruplet changes (Mantis #33435)
-----------------------------------------------------------------------
create or replace 
TRIGGER TBIU_MEASUREMENT_FILE before insert or update
on MEASUREMENT_FILE for each row
declare
	nb_count number;
begin
	-- Trigger permettant d'avoir directement les ID des données IN SITU sur lesquelles sont les résultats
	case :new.OBJECT_TYPE_CD 

    when 'PASS' then
      :new.SURVEY_ID := :new.OBJECT_ID ;
  
    when 'PREL' then
    begin
      :new.SAMPLING_OPER_ID := :new.OBJECT_ID ;
      select count(SURVEY_ID) into nb_count from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.OBJECT_ID ;
      if (nb_count = 1) then
        select SURVEY_ID into :new.SURVEY_ID from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.OBJECT_ID ;
      end if;
    end ;
  
    when 'ECHANT' then
    begin
      :new.SAMPLE_ID := :new.OBJECT_ID ;
      select count(SAMPLING_OPER_ID) into nb_count from SAMPLE where SAMPLE_ID = :new.OBJECT_ID ;
      if (nb_count = 1) then 
        select SAMPLING_OPER_ID into :new.SAMPLING_OPER_ID from SAMPLE where SAMPLE_ID = :new.OBJECT_ID ; 
      end if;
      select count(SURVEY_ID) into nb_count from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.SAMPLING_OPER_ID ;
      if (nb_count = 1) then
        select SURVEY_ID into :new.SURVEY_ID from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.SAMPLING_OPER_ID ;
      end if;
    end ;

	end case;

  -- Make sure PMFM_ID is filled and valid (need for ReefDB synchro-server)
  if (:new.PMFM_ID is null
      -- also update PMFM_ID if quadruplet changes (Mantis #33435)
      or :old.PAR_CD != :new.PAR_CD or :old.MATRIX_ID != :new.MATRIX_ID or :old.FRACTION_ID != :new.FRACTION_ID or :old.METHOD_ID != :new.METHOD_ID) then
    begin
      select
        PMFM_ID into :new.PMFM_ID
      from PMFM
      where
        PAR_CD = :new.PAR_CD
        and MATRIX_ID = :new.MATRIX_ID
        and FRACTION_ID = :new.FRACTION_ID
        and METHOD_ID = :new.METHOD_ID;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          raise_application_error(-20101, 'PAR_CD='||:new.PAR_CD||', MATRIX_ID='||:new.MATRIX_ID||', FRACTION_ID='||:new.FRACTION_ID||', METHOD_ID='||:new.METHOD_ID||' not exists in table PMFM.');
    end;
  end if;

end;
/

-----------------------------------------------------------------------
-- PMFM_CONTEXT_ORDER
-- THIS TRIGGER IS REPLACED BY TBIU_PMFM_CONTEXT_ORDER (Mantis #320027)
-----------------------------------------------------------------------
-- Insert
--create or replace
--trigger TBI_PMFM_CONTEXT_ORDER before insert
--on PMFM_CONTEXT_ORDER for each row
--when (new.PMFM_ID is null)
--begin
--  select
--    PMFM_ID into :new.PMFM_ID
--  from PMFM
--  where
--    PAR_CD = :new.PAR_CD
--    and MATRIX_ID = :new.MATRIX_ID
--    and FRACTION_ID = :new.FRACTION_ID
--    and METHOD_ID = :new.METHOD_ID;
--end;
--/

-----------------------------------------------------------------------
-- PMFM_CONTEXT_ORDER
-----------------------------------------------------------------------
CREATE OR REPLACE TRIGGER TBIU_PMFM_CONTEXT_ORDER before insert or update
on PMFM_CONTEXT_ORDER for each row
begin

  if (:new.PMFM_ID is null) then
    select
      PMFM_ID into :new.PMFM_ID
    from PMFM
    where
      PAR_CD = :new.PAR_CD
      and MATRIX_ID = :new.MATRIX_ID
      and FRACTION_ID = :new.FRACTION_ID
      and METHOD_ID = :new.METHOD_ID;
  end if;

  -- Ensure that PMFM tuple is valid (Mantis #320027)
  if (:new.PMFM_ID is not null AND
      :new.PAR_CD is not null AND
      :new.MATRIX_ID is not null AND
      :new.FRACTION_ID is not null AND
      :new.METHOD_ID is not null) then
    DECLARE
      pmfmId PMFM.PMFM_ID%TYPE;
    BEGIN
      SELECT PMFM_ID INTO pmfmId FROM PMFM
      WHERE PMFM_ID = :new.PMFM_ID
      AND PAR_CD = :new.PAR_CD
      AND MATRIX_ID = :new.MATRIX_ID
      AND FRACTION_ID = :new.FRACTION_ID
      AND METHOD_ID = :new.METHOD_ID;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        raise_application_error(-20101, 'PMFM_ID='||:new.PMFM_ID||', PAR_CD='||:new.PAR_CD||', MATRIX_ID='||:new.MATRIX_ID||', FRACTION_ID='||:new.FRACTION_ID||', METHOD_ID='||:new.METHOD_ID||' not exists in table PMFM.');
    END;
  end if;

end;
/


-----------------------------------------------------------------------
-- PMFM_STRAT_ACQUIS_LEVEL
-----------------------------------------------------------------------
create or replace
trigger TBIU_PMFM_STRAT_ACQUIS_LEVEL before insert or update
on PMFM_STRAT_ACQUIS_LEVEL for each row
begin
  -- Trigger permettant de remplir les colonnes obligatoires manquantes
  -- quand les insertions sont faites par le serveur de synchro
  if ((:new.PMFM_STRAT_ID IS NOT NULL) 
      AND ((:new.STRAT_ID IS NULL) or (:new.PAR_CD is null) OR (:new.MATRIX_ID is null) OR (:new.FRACTION_ID is null) OR (:new.METHOD_ID is null))) then
    select
      STRAT_ID, PAR_CD, MATRIX_ID, FRACTION_ID, METHOD_ID
      into :new.STRAT_ID, :new.PAR_CD, :new.MATRIX_ID, :new.FRACTION_ID, :new.METHOD_ID
    from PMFM_STRATEGY
    where PMFM_STRAT_ID = :new.PMFM_STRAT_ID;
  end if;
  
  if (:new.PMFM_STRAT_ID IS NULL) then
    select      
      PMFM_STRAT_ID
      into :new.PMFM_STRAT_ID
    from PMFM_STRATEGY
    where 
      STRAT_ID = :new.STRAT_ID
      AND PAR_CD = :new.PAR_CD
      AND MATRIX_ID = :new.MATRIX_ID
      AND FRACTION_ID = :new.FRACTION_ID
      AND METHOD_ID = :new.METHOD_ID;
  end if;

  -- Ensure that PMFM tuple is valid (Mantis #320027)
  if (:new.PMFM_STRAT_ID is not null AND
      :new.PAR_CD is not null AND
      :new.MATRIX_ID is not null AND
      :new.FRACTION_ID is not null AND
      :new.METHOD_ID is not null) then
    DECLARE
      pmfmStratId PMFM_STRATEGY.PMFM_STRAT_ID%TYPE;
    BEGIN
      SELECT PMFM_STRAT_ID INTO pmfmStratId FROM PMFM_STRATEGY
      WHERE PMFM_STRAT_ID = :new.PMFM_STRAT_ID
      AND PAR_CD = :new.PAR_CD
      AND MATRIX_ID = :new.MATRIX_ID
      AND FRACTION_ID = :new.FRACTION_ID
      AND METHOD_ID = :new.METHOD_ID;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        raise_application_error(-20101, 'PMFM_STRAT_ID='||:new.PMFM_STRAT_ID||', PAR_CD='||:new.PAR_CD||', MATRIX_ID='||:new.MATRIX_ID||', FRACTION_ID='||:new.FRACTION_ID||', METHOD_ID='||:new.METHOD_ID||' not exists in table PMFM_STRATEGY.');
    END;
  end if;

end;
/


-----------------------------------------------------------------------
-- PMFM_STRAT_UI_FUNCTION
-----------------------------------------------------------------------
create or replace
trigger TBIU_PMFM_STRAT_UI_FUNCTION before insert or update
on PMFM_STRAT_UI_FUNCTION for each row
begin
  -- Trigger permettant de remplir les colonnes obligatoires manquantes
  -- quand les insertions sont faites par le serveur de synchro
  if ((:new.PMFM_STRAT_ID IS NOT NULL) 
      AND ((:new.STRAT_ID IS NULL) OR (:new.PAR_CD is null) OR (:new.MATRIX_ID is null) OR (:new.FRACTION_ID is null) OR (:new.METHOD_ID is null))) then
    select
    STRAT_ID, PAR_CD, MATRIX_ID, FRACTION_ID, METHOD_ID
    into :new.STRAT_ID, :new.PAR_CD, :new.MATRIX_ID, :new.FRACTION_ID, :new.METHOD_ID
    from PMFM_STRATEGY
    where PMFM_STRAT_ID = :new.PMFM_STRAT_ID;
  end if;
  
  if (:new.PMFM_STRAT_ID IS NULL) then
    select      
      PMFM_STRAT_ID
      into :new.PMFM_STRAT_ID
    from PMFM_STRATEGY
    where 
      STRAT_ID = :new.STRAT_ID
      AND PAR_CD = :new.PAR_CD
      AND MATRIX_ID = :new.MATRIX_ID
      AND FRACTION_ID = :new.FRACTION_ID
      AND METHOD_ID = :new.METHOD_ID;
  end if;

  -- Ensure that PMFM tuple is valid (Mantis #320027)
  if (:new.PMFM_STRAT_ID is not null AND
      :new.PAR_CD is not null AND
      :new.MATRIX_ID is not null AND
      :new.FRACTION_ID is not null AND
      :new.METHOD_ID is not null) then
    DECLARE
      pmfmStratId PMFM_STRATEGY.PMFM_STRAT_ID%TYPE;
    BEGIN
      SELECT PMFM_STRAT_ID INTO pmfmStratId FROM PMFM_STRATEGY
      WHERE PMFM_STRAT_ID = :new.PMFM_STRAT_ID
      AND PAR_CD = :new.PAR_CD
      AND MATRIX_ID = :new.MATRIX_ID
      AND FRACTION_ID = :new.FRACTION_ID
      AND METHOD_ID = :new.METHOD_ID;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        raise_application_error(-20101, 'PMFM_STRAT_ID='||:new.PMFM_STRAT_ID||', PAR_CD='||:new.PAR_CD||', MATRIX_ID='||:new.MATRIX_ID||', FRACTION_ID='||:new.FRACTION_ID||', METHOD_ID='||:new.METHOD_ID||' not exists in table PMFM_STRATEGY.');
    END;
  end if;

end;
/

-----------------------------------------------------------------------
-- TAXON_MEASUREMENT
-- LP 03/01/2017: update PMFM_ID if quadruplet changes (Mantis #33435)
-----------------------------------------------------------------------
create or replace 
trigger "TBIU_TAXON_MEASUREMENT" before insert or update
on TAXON_MEASUREMENT for each row
declare
	nb_count number;
begin
	-- Trigger permettant d'avoir directement les ID des données IN SITU sur lesquelles sont les résultats
	case :new.OBJECT_TYPE_CD 

    when 'PASS' then
      :new.SURVEY_ID := :new.OBJECT_ID ;
  
    when 'PREL' then
    begin
      :new.SAMPLING_OPER_ID := :new.OBJECT_ID ;
      select count(SURVEY_ID) into nb_count from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.OBJECT_ID ;
      if (nb_count = 1) then
        select SURVEY_ID into :new.SURVEY_ID from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.OBJECT_ID ;
      end if;
    end ;
  
    when 'ECHANT' then
    begin
      :new.SAMPLE_ID := :new.OBJECT_ID ;
      select count(SAMPLING_OPER_ID) into nb_count from SAMPLE where SAMPLE_ID = :new.OBJECT_ID ;
      if (nb_count = 1) then 
        select SAMPLING_OPER_ID into :new.SAMPLING_OPER_ID from SAMPLE where SAMPLE_ID = :new.OBJECT_ID ; 
      end if;
      select count(SURVEY_ID) into nb_count from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.SAMPLING_OPER_ID ;
      if (nb_count = 1) then
        select SURVEY_ID into :new.SURVEY_ID from SAMPLING_OPERATION where SAMPLING_OPER_ID = :new.SAMPLING_OPER_ID ;
      end if;
    end ;

	end case;

  -- Make sure PMFM_ID is filled and valid (need for ReefDB synchro-server)
  if (:new.PMFM_ID is null
      -- also update PMFM_ID if quadruplet changes (Mantis #33435)
      or :old.PAR_CD != :new.PAR_CD or :old.MATRIX_ID != :new.MATRIX_ID or :old.FRACTION_ID != :new.FRACTION_ID or :old.METHOD_ID != :new.METHOD_ID) then
    begin
      select
        PMFM_ID into :new.PMFM_ID
      from PMFM
      where
        PAR_CD = :new.PAR_CD
        and MATRIX_ID = :new.MATRIX_ID
        and FRACTION_ID = :new.FRACTION_ID
        and METHOD_ID = :new.METHOD_ID;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          raise_application_error(-20101, 'PAR_CD='||:new.PAR_CD||', MATRIX_ID='||:new.MATRIX_ID||', FRACTION_ID='||:new.FRACTION_ID||', METHOD_ID='||:new.METHOD_ID||' not exists in table PMFM.');
    end;
  end if;

end;
/


-----------------------------------------------------------------------
-- MON_LOC_PMFM_MET
-----------------------------------------------------------------------
create or replace 
trigger "TBI_MON_LOC_PMFM_MET" before insert
on MON_LOC_PMFM_MET for each row
when (new.MON_LOC_MET_ID is null)
begin
  select
    MON_LOC_MET_ID into :new.MON_LOC_MET_ID
  from MON_LOC_MET
  where 
    MET_CD = :new.MET_CD
    and MON_LOC_ID = :new.MON_LOC_ID;
end;
/

-----------------------------------------------------------------------
-- MON_LOC_MET
-----------------------------------------------------------------------
create or replace
trigger "TBI_MON_LOC_MET" before insert
on MON_LOC_MET for each row
when (new.MON_LOC_MET_ID is null)
begin
  select
    SEQ_MON_LOC_MET_ID.nextval into :new.MON_LOC_MET_ID
  from dual;
end;
/


-----------------------------------------------------------------------
-- DEFAULT_VALUE
-----------------------------------------------------------------------
create or replace 
trigger TBD_ORDER_ITEM before delete on ORDER_ITEM
for each row 
BEGIN 
  BEGIN 
    INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.ORDER_ITEM_ID, systimestamp, 'ORDER_ITEM'); 
    EXCEPTION WHEN OTHERS THEN 
      INSERT INTO OBJECT_TYPE (OBJECT_TYPE_CD, OBJECT_TYPE_NM, UPDATE_DT) values('ORDER_ITEM', 'Table ORDER_ITEM', systimestamp); 
      INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.ORDER_ITEM_ID, systimestamp, 'ORDER_ITEM'); 
  END;
END;
/

create or replace 
trigger "TBI_ORDER_ITEM" before insert
on ORDER_ITEM for each row
when (new.ORDER_ITEM_ID is null)
begin
  select
    SEQ_ORDER_ITEM_ID.nextval into :new.ORDER_ITEM_ID
  from dual;
end;
/

-----------------------------------------------------------------------
-- MON_LOC_ORDER_ITEM
-----------------------------------------------------------------------
create or replace 
trigger "TBI_MON_LOC_ORDER_ITEM" before insert
on MON_LOC_ORDER_ITEM for each row
when (new.ORDER_ITEM_ID is null)
begin
  select
    ORDER_ITEM_ID into :new.ORDER_ITEM_ID
  from ORDER_ITEM
  where 
    ORDER_ITEM_TYPE_CD = :new.ORDER_ITEM_TYPE_CD
    and ORDER_ITEM_CD = :new.ORDER_ITEM_CD;
end;
/

-----------------------------------------------------------------------
-- MON_LOC_ORDER_ITEM
-----------------------------------------------------------------------
create or replace 
trigger "TBI_MON_LOC_PROG" before insert
on MON_LOC_PROG for each row
when (new.MON_LOC_PROG_ID is null)
begin
  select
    SEQ_MON_LOC_PROG_ID.nextval into :new.MON_LOC_PROG_ID
  from dual;
end;
/

-----------------------------------------------------------------------
-- MOR_MON_LOC_PROG
-----------------------------------------------------------------------
create or replace 
trigger "TBI_MOR_MON_LOC_PROG" before insert
on MOR_MON_LOC_PROG for each row
when (new.MON_LOC_PROG_ID is null)
begin
  select
    MON_LOC_PROG_ID into :new.MON_LOC_PROG_ID
  from MON_LOC_PROG
  where 
    MON_LOC_ID = :new.MON_LOC_ID
    and PROG_CD = :new.PROG_CD;
end;
/

-----------------------------------------------------------------------
-- Program/Strategy tables
-----------------------------------------------------------------------
-- PROGRAM Delete
create or replace 
trigger TBD_PROGRAMME before delete 
on PROGRAMME for each row 
begin
  insert into DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_CD, UPDATE_DT, OBJECT_TYPE_CD)
  values(SEQ_DEL_ITEM_HIST_ID.nextval, :old.PROG_CD, systimestamp, 'PROGRAMME');
end;
/
-- STRATEGY Delete
create or replace 
trigger TBD_STRATEGY before delete 
on STRATEGY for each row 
begin
  insert into DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD)
  values(SEQ_DEL_ITEM_HIST_ID.nextval, :old.STRAT_ID, systimestamp, 'STRATEGY');
end;
/
-- PMFM_STRATEGY Delete
create or replace 
trigger TBD_PMFM_STRATEGY before delete 
on PMFM_STRATEGY for each row 
begin
  insert into DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD)
  values(SEQ_DEL_ITEM_HIST_ID.nextval, :old.PMFM_STRAT_ID, systimestamp, 'PMFM_STRATEGY');
end;
/
-- APPLIED_STRATEGY Delete
create or replace 
trigger TBD_APPLIED_STRATEGY before delete 
on APPLIED_STRATEGY for each row 
begin
  insert into DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD)
  values(SEQ_DEL_ITEM_HIST_ID.nextval, :old.APPLIED_STRAT_ID, systimestamp, 'APPLIED_STRATEGY');
end;
/
-- PMFM_APPLIED_STRATEGY Delete
create or replace 
trigger TBD_PMFM_APPLIED_STRATEGY before delete 
on PMFM_APPLIED_STRATEGY for each row 
begin
  insert into DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_CD, UPDATE_DT, OBJECT_TYPE_CD)
  values(SEQ_DEL_ITEM_HIST_ID.nextval, :old.PMFM_STRAT_ID || '~~'||:old.APPLIED_STRAT_ID, systimestamp, 'PMFM_APPLIED_STRATEGY');
end;
/
-- APPLIED_PERIOD Delete
create or replace 
trigger TBD_APPLIED_PERIOD before delete 
on APPLIED_PERIOD for each row 
begin
  insert into DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_CD, UPDATE_DT, OBJECT_TYPE_CD)
  values(SEQ_DEL_ITEM_HIST_ID.nextval, to_char(:old.APPLIED_PERIOD_START_DT, 'yyyy-mm-dd hh:mm:ss.SSSSS') || '~~'||:old.APPLIED_STRAT_ID, systimestamp, 'APPLIED_PERIOD');
end;
/

-- ---------------------------------------------------------------------
-- SURVEY
-- ---------------------------------------------------------------------
create or replace 
trigger TBD_SURVEY before delete 
on SURVEY for each row 
begin
  insert into DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD)
  values(SEQ_DEL_ITEM_HIST_ID.nextval, :old.SURVEY_ID, systimestamp, 'PASS');
end;
/

-- Add trigger to update 'update_dt' on each inserted/modified row of SURVEY (Mantis #33261)
create or replace
trigger TBIU_SURVEY_UD before insert or update
on SURVEY for each row
when ((new.UPDATE_DT is null) or (new.UPDATE_DT = old.UPDATE_DT))
begin
  :new.UPDATE_DT := systimestamp;
end;
/

-----------------------------------------------------------------------
-- SANDRE_PMFMU_EXP
--  29/02/2016 BL  Add trigger TBIU_SANDRE_PMFMU_EXP (fix mantis #29269)
-----------------------------------------------------------------------
create or replace
trigger TBIU_SANDRE_PMFMU_EXP before insert or update
on SANDRE_PMFMU_EXP for each row
when (new.PMFM_ID is null)
begin
  select
    PMFM_ID into :new.PMFM_ID
  from PMFM
  where
    PAR_CD = :new.PAR_CD
    and MATRIX_ID = :new.MATRIX_ID
    and FRACTION_ID = :new.FRACTION_ID
    and METHOD_ID = :new.METHOD_ID;
end;
/

-----------------------------------------------------------------------
-- SANDRE_PMFMU_IMP
--  06/04/2016 LPT  Add trigger TBIU_SANDRE_PMFMU_IMP
-----------------------------------------------------------------------
create or replace
trigger TBIU_SANDRE_PMFMU_IMP before insert or update
on SANDRE_PMFMU_IMP for each row
when (new.PMFM_ID is null)
begin
  select
    PMFM_ID into :new.PMFM_ID
  from PMFM
  where
    PAR_CD = :new.PAR_CD
    and MATRIX_ID = :new.MATRIX_ID
    and FRACTION_ID = :new.FRACTION_ID
    and METHOD_ID = :new.METHOD_ID;
end;
/

-----------------------------------------------------------------------
-- CAMPAIGN
-- 17/05/2016 LP  Add trigger on deletion (CAMPAIGN, OCCASION)
-----------------------------------------------------------------------
create or replace
trigger TBD_CAMPAIGN before delete on CAMPAIGN
for each row
BEGIN
  BEGIN
    INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.CAMPAIGN_ID, systimestamp, 'CAMP');
    EXCEPTION WHEN OTHERS THEN
      INSERT INTO OBJECT_TYPE (OBJECT_TYPE_CD, OBJECT_TYPE_NM, UPDATE_DT) values('CAMP', 'Table CAMPAIGN', systimestamp);
      INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.CAMPAIGN_ID, systimestamp, 'CAMP');
  END;
END;
/
create or replace
trigger TBD_OCCASION before delete on OCCASION
for each row
BEGIN
  BEGIN
    INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.OCCAS_ID, systimestamp, 'SORTIE');
    EXCEPTION WHEN OTHERS THEN
      INSERT INTO OBJECT_TYPE (OBJECT_TYPE_CD, OBJECT_TYPE_NM, UPDATE_DT) values('SORTIE', 'Table OCCASION', systimestamp);
      INSERT INTO DELETED_ITEM_HISTORY (DEL_ITEM_HIST_ID, OBJECT_ID, UPDATE_DT, OBJECT_TYPE_CD) VALUES(SEQ_DEL_ITEM_HIST_ID.nextval, :old.OCCAS_ID, systimestamp, 'SORTIE');
  END;
END;
/

-- 07/04/2017 LP  Add trigger on quser_privilege (mantis #34994)
create or replace
trigger TBIU_QUSER_PRIVILEGE before insert or update
on QUSER_PRIVILEGE for each row
begin
    update QUSER set UPDATE_DT = systimestamp where QUSER_ID = :new.QUSER_ID;
end;
/
create or replace
trigger TBD_QUSER_PRIVILEGE before delete
on QUSER_PRIVILEGE for each row
begin
    update QUSER set UPDATE_DT = systimestamp where QUSER_ID = :old.QUSER_ID;
end;
/
